@@ -4,6 +4,9 @@ pub use crate::graph::csr::CsrLayout;
 pub use crate::graph::csr::DirectedCsrGraph;
 pub use crate::graph::csr::UndirectedCsrGraph;
 
+pub use crate::graph::labeled::DirectedNodeLabeledCsrGraph;
+pub use crate::graph::labeled::UndirectedNodeLabeledCsrGraph;
+
 pub use crate::graph_ops::DegreePartitionOp;
 pub use crate::graph_ops::ForEachNodeParallelByPartitionOp;
 pub use crate::graph_ops::ForEachNodeParallelOp;
@@ -11,10 +14,14 @@ pub use crate::graph_ops::InDegreePartitionOp;
 pub use crate::graph_ops::OutDegreePartitionOp;
 pub use crate::graph_ops::RelabelByDegreeOp;
 
+pub use crate::index::AtomicIdx;
 pub use crate::index::Idx;
 
-pub use crate::input::edgelist::EdgeListInput;
+pub use crate::input::*;
 
 pub use crate::DirectedGraph;
 pub use crate::Graph;
+pub use crate::NodeLabeledGraph;
 pub use crate::UndirectedGraph;
+
+pub use crate::Error;
